import { backend } from "./backend.js";
import { authenticating, currentUserMode, isRendLimitReached, now, refresh, refreshDynStates, refreshScores, scoreDefsModule, setRendState } from "./webapp.js";
import { computeOnChange, fixupScore, isScoreDataComputable, isScoreDefValuable, newScore, normScale } from "./scores.js";
import { DIALOG } from "./dialog.js";
export class BackendFacade {
    constructor() {
        this.real = backend;
        this.user = null;
    }
    async initialize(front) {
        if (!this.real)
            try {
                this.real = (await import("../skin/backend.js")).backend || {};
            }
            catch (e) {
                this.real = {};
            }
        this.rend = new BackendRendFacade(front, this);
        this.scorm = new BackendScormFacade(front, this);
        this.rendSynthesis = new BackendRendSynthFacade(front, this);
        this.scores = new BackendScoresFacade(front, this);
        if (this.real.initialize) {
            this.backendProps = (await this.real.initialize(front)) || {};
            if ('neverStoreLocaly' in this.backendProps) {
                this.backendProps.rendNeverLocalyStored = this.backendProps.neverStoreLocaly;
                this.backendProps.scoreNeverLocalyStored = this.backendProps.neverStoreLocaly;
                this.backendProps.resIONeverLocalyStored = this.backendProps.neverStoreLocaly;
            }
        }
        else {
            this.backendProps = {};
        }
    }
    async tryAuthenticate() {
        var _a, _b;
        await ((_b = (_a = this.real.auth) === null || _a === void 0 ? void 0 : _a.tryAuthenticate) === null || _b === void 0 ? void 0 : _b.call(this.real.auth));
        return this.onLoginChange();
    }
    isUserAdmin() {
        var _a, _b;
        return ((_b = (_a = this.user) === null || _a === void 0 ? void 0 : _a.perms) === null || _b === void 0 ? void 0 : _b.indexOf('admin')) >= 0 || false;
    }
    isUserCorrector() {
        var _a, _b;
        return ((_b = (_a = this.user) === null || _a === void 0 ? void 0 : _a.perms) === null || _b === void 0 ? void 0 : _b.indexOf('corrector')) >= 0 || false;
    }
    async onLoginChange() {
        await this.fetchUser();
        await this.rend.init();
        refresh();
        this.scores.fetchScores();
    }
    async fetchUser() {
        var _a, _b;
        try {
            if ((_b = (_a = this.real) === null || _a === void 0 ? void 0 : _a.auth) === null || _b === void 0 ? void 0 : _b.userProfile) {
                this.user = await this.real.auth.userProfile();
                this.updtUser();
            }
        }
        catch (e) {
            console.log(e);
            this.user = null;
        }
        return this.user;
    }
    resetUser() {
        this.user = null;
        this.updtUser();
    }
    updtUser() {
        var _a, _b;
        if (!this.user) {
            currentUserMode.solVisible = false;
            currentUserMode.contentVisible = false;
            currentUserMode.rendEditable = false;
            currentUserMode.asCorrector = false;
        }
        else {
            if (((_a = this.user.perms) === null || _a === void 0 ? void 0 : _a.indexOf('admin')) < 0) {
                currentUserMode.solVisible = false;
                currentUserMode.contentVisible = false;
                currentUserMode.rendEditable = false;
            }
            if (((_b = this.user.perms) === null || _b === void 0 ? void 0 : _b.indexOf('corrector')) >= 0) {
                const storeUserMode = localStorage === null || localStorage === void 0 ? void 0 : localStorage.getItem("par:currentUserMode");
                if (storeUserMode)
                    try {
                        const data = JSON.parse(storeUserMode);
                        if (data.ts > Date.now() - 3600000) {
                            currentUserMode.asCorrector = data.asCorrector;
                            currentUserMode.solVisible = data.solVisible;
                            currentUserMode.contentVisible = data.contentVisible;
                            currentUserMode.rendEditable = data.rendEditable;
                            return;
                        }
                    }
                    catch (e) { }
                currentUserMode.asCorrector = true;
            }
        }
    }
    async fetchCohort(page) {
        var _a, _b;
        return ((_b = (_a = this.real) === null || _a === void 0 ? void 0 : _a.cohort) === null || _b === void 0 ? void 0 : _b.fetchCohort(page)) || Promise.resolve({ users: [] });
    }
    getEmail(user) {
        if (this.backendProps.userEmailField === "pseudo")
            return user.pseudo;
        return user.email || "";
    }
    isSendMsgAvailable() {
        return this.backendProps.userEmailField != null;
    }
    async sendMsgTo(to, subject) {
        if (to.users.length === 1) {
            window.open(`mailto:${this.getEmail(to.users[0])}?subject=${encodeURIComponent(subject || "")}`);
        }
        else {
            const mails = to.users.map((u) => this.getEmail(u)).join(',');
            window.open(`mailto:?bcc=${mails}&subject=${encodeURIComponent(subject || "")}`);
        }
    }
}
export class BackendRendFacade {
    constructor(front, main) {
        this.validated = {};
        this.front = front;
        this.main = main;
    }
    async init() {
        var _a;
        const rend = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend;
        let backendValidated = (rend === null || rend === void 0 ? void 0 : rend.listValidated) ? await rend.listValidated() : null;
        if (this.main.backendProps.rendNeverLocalyStored !== true) {
            const db = await getDb();
            if (db) {
                if (backendValidated == null) {
                    backendValidated = {};
                    const rendArray = await doRequest(db.transaction(DB_rendValid).objectStore(DB_rendValid).getAll());
                    if (rendArray)
                        for (const r of rendArray) {
                            backendValidated[r.rendId] = r;
                        }
                }
                else {
                    const tr = db.transaction(DB_rendValid, 'readwrite');
                    const rendValid = tr.objectStore(DB_rendValid);
                    rendValid.clear();
                    for (const rendId in this.validated) {
                        const rend = this.validated[rendId];
                        rendValid.put({ rendId, ts: rend === null || rend === void 0 ? void 0 : rend.ts, clientTs: rend === null || rend === void 0 ? void 0 : rend.clientTs });
                    }
                    await doCommit(tr);
                }
            }
        }
        this.validated = backendValidated || {};
    }
    async hasLocalData() {
        const db = await getDb();
        if (db) {
            const tr = db.transaction([DB_rendFields, DB_rendValid], "readonly");
            return await doRequest(tr.objectStore(DB_rendFields).count()) > 0 || await doRequest(tr.objectStore(DB_rendValid).count()) > 0;
        }
        return false;
    }
    async clearLocalData() {
        this.validated = {};
        const db = await getDb();
        if (db) {
            const tr = db.transaction([DB_rendFields, DB_rendValid], "readwrite");
            tr.objectStore(DB_rendFields).clear();
            tr.objectStore(DB_rendValid).clear();
            return doCommit(tr);
        }
    }
    async getLocalStoreInfo(schema) {
        var _a, _b;
        if (this.main.backendProps.rendNeverLocalyStored === true)
            return null;
        const remoteManage = (_b = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend) === null || _b === void 0 ? void 0 : _b.canManage(schema);
        if (!remoteManage || !this.isServerOnline()) {
            let st = false;
            try {
                st = localStorage.getItem(LS_RendLocalStore) !== "false";
            }
            catch (e) {
                return null;
            }
            if (await getDb() == null)
                return null;
            return {
                lsState: st,
                lsOptionLabel: !remoteManage ? "Enregistrer dans mon navigateur" : "Enregistrer dans mon navigateur tant que je suis hors-ligne",
                onLsOptionChange: (state) => {
                    try {
                        localStorage.setItem(LS_RendLocalStore, state ? "true" : "false");
                    }
                    catch (e) {
                        return false;
                    }
                    return state;
                }
            };
        }
        return null;
    }
    canRemoteValidate(schema) {
        var _a;
        if (this.main.user == null)
            return false;
        const rend = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend;
        return rend && ('validate' in rend) && (rend.canManage ? rend.canManage(schema) : true);
    }
    canRemoteRevoke(schema) {
        var _a;
        if (this.main.user == null)
            return false;
        const rend = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend;
        return rend && ('revoke' in rend) && (rend.canManage ? rend.canManage(schema) : true);
    }
    canRemoteSaveDraft(schema) {
        var _a;
        if (this.main.user == null)
            return false;
        const rend = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend;
        return rend && ('saveDraft' in rend) && (rend.canManage ? rend.canManage(schema) : true);
    }
    async onFieldChange(schema, field, fieldKey, value, localStoreAllowed) {
        if (this.isServerOnline() && this.canRemoteSaveDraft(schema)) {
            const rend = this.main.real.rend;
            if (value == null && 'deleteInDraft' in rend) {
                await rend.deleteInDraft(fieldKey, schema);
                return true;
            }
            const data = new FormData();
            data.set(fieldKey, value == null ? (field.jsType === "File" ? emptyFile : "") : value);
            await rend.saveDraft(data, schema);
            return true;
        }
        if (localStoreAllowed && this.main.backendProps.rendNeverLocalyStored !== true) {
            const db = await getDb();
            if (db) {
                const tr = db.transaction(DB_rendFields, 'readwrite');
                if (value == null) {
                    tr.objectStore(DB_rendFields).delete([schema.rendId, fieldKey]);
                }
                else {
                    const rendField = {
                        rendId: schema.rendId,
                        fieldKey: fieldKey,
                        value: value,
                        clientTs: Date.now()
                    };
                    tr.objectStore(DB_rendFields).put(rendField);
                }
                await doCommit(tr);
                return true;
            }
        }
        return false;
    }
    async validate(data, schema) {
        if (this.isServerOnline() && this.canRemoteValidate(schema)) {
            const r = await this.main.real.rend.validate(data, schema);
            this.validated[schema.rendId] = r;
            this.dispatchRendChange(schema.rendId, r);
        }
        else if (this.main.backendProps.rendNeverLocalyStored !== true) {
            const db = await getDb();
            if (db) {
                const rendValid = {
                    rendId: schema.rendId,
                };
                const tr = db.transaction(DB_rendValid, 'readwrite');
                tr.objectStore(DB_rendValid).put(rendValid);
                await doCommit(tr);
                this.validated[schema.rendId] = rendValid;
                this.dispatchRendChange(schema.rendId, rendValid);
            }
        }
    }
    async revoke(schema) {
        if (this.isServerOnline() && this.canRemoteRevoke(schema)) {
            await this.main.real.rend.revoke(schema);
            this.validated[schema.rendId] = undefined;
            this.dispatchRendChange(schema.rendId, null);
        }
        else if (this.main.backendProps.rendNeverLocalyStored !== true) {
            const db = await getDb();
            if (db) {
                const tr = db.transaction(DB_rendValid, 'readwrite');
                tr.objectStore(DB_rendValid).delete([schema.rendId]);
                await doCommit(tr);
                this.validated[schema.rendId] = undefined;
                this.dispatchRendChange(schema.rendId, null);
            }
        }
    }
    isValidated(rendId) {
        return this.validated[rendId];
    }
    async fetchLastFields(schema) {
        var _a, _b;
        const rend = (_a = this.main.real) === null || _a === void 0 ? void 0 : _a.rend;
        const fields = new StoredRendFieldsImpl(schema);
        if (((_b = rend === null || rend === void 0 ? void 0 : rend.canManage) === null || _b === void 0 ? void 0 : _b.call(rend, schema)) && rend.fetchLastFields) {
            await rend.fetchLastFields(schema, fields);
            return fields;
        }
        if (this.main.backendProps.rendNeverLocalyStored !== true) {
            const db = await getDb();
            if (db) {
                const fieldsStore = db.transaction(DB_rendFields).objectStore(DB_rendFields);
                await doCursor(fieldsStore.openCursor(IDBKeyRange.bound([schema.rendId, ""], [schema.rendId, "~~~~~"], true, true)), (cursor) => {
                    const field = cursor.value;
                    if (field.value instanceof File) {
                        fields.setFieldBlob(field.fieldKey, field.value.name, field.value, field.ts, field.clientTs);
                    }
                    else {
                        fields.setFieldString(field.fieldKey, field.value, field.ts, field.clientTs);
                    }
                    cursor.continue();
                });
                return fields;
            }
        }
        return null;
    }
    dispatchRendChange(rendId, state) {
        const isLimitReached = isRendLimitReached(rendId);
        refreshDynStates(`[p-rendid="${rendId}"]`, (tag) => {
            setRendState(tag, state, isLimitReached);
        });
    }
    isSynchroAvailable() {
        return false;
    }
    getOfflineSynchroState() {
        return 'off';
    }
    isServerOnline() {
        return this.getOfflineSynchroState() !== 'recording';
    }
    buildMsgNoStorageAvailable(msgRoot) {
        if (this.main.real.rend.buildMsgNoStorageAvailable) {
            this.main.real.rend.buildMsgNoStorageAvailable(msgRoot);
        }
        else {
            msgRoot.append("Ce rendu n'est pas enregistré");
        }
        return msgRoot;
    }
}
class BackendScormFacade {
    constructor(front, main) {
        this.main = main;
    }
    async open(resId, options) {
        var _a, _b, _c, _d;
        if (authenticating)
            await authenticating;
        try {
            (_a = window.API_1484_11) === null || _a === void 0 ? void 0 : _a.Terminate("");
            (_b = window.API) === null || _b === void 0 ? void 0 : _b.LMSFinish("");
            let api12 = null;
            let api2k4 = null;
            const real = this.main.real;
            if ((_c = real === null || real === void 0 ? void 0 : real.scorm) === null || _c === void 0 ? void 0 : _c.open2k4) {
                const apis = await real.scorm.open2k4(resId, options);
                api12 = apis.scorm12;
                api2k4 = apis.scorm2k4;
            }
            else if ((_d = real === null || real === void 0 ? void 0 : real.scorm) === null || _d === void 0 ? void 0 : _d.open12) {
                api12 = await real.scorm.open12(resId, options);
            }
            if ((!api12 || !api2k4) && !this.main.backendProps.resIONeverLocalyStored) {
                const db = await getDb();
                if (db) {
                    const localApi = api2k4 = new (await import("./localScorm.js")).LocalScormAPI();
                    await localApi.internalInit(db, resId, options);
                    if (!api12)
                        api12 = localApi;
                    if (!api2k4)
                        api2k4 = localApi;
                }
            }
            if (api12) {
                api12 = Object.create(api12);
                api12.LMSFinish = function (p) {
                    window.API = null;
                    return Object.getPrototypeOf(this).LMSFinish.call(this, p);
                };
                api12.LMSCommit = (p) => {
                    if (!api12.noStore)
                        this.saveGlobalScore(resId, api12.LMSGetValue("cmi.core.score.min"), api12.LMSGetValue("cmi.core.score.raw"), api12.LMSGetValue("cmi.core.score.max"), null, null);
                    return Object.getPrototypeOf(api12).LMSCommit.call(api12, p);
                };
                window.API = api12;
            }
            if (api2k4) {
                api2k4 = Object.create(api2k4);
                api2k4.Terminate = function (p) {
                    window.API_1484_11 = null;
                    return Object.getPrototypeOf(this).Terminate.call(this, p);
                };
                api2k4.Commit = (p) => {
                    if (!api2k4.noStore)
                        this.saveGlobalScore(resId, api2k4.GetValue("cmi.score.min"), api2k4.GetValue("cmi.score.raw"), api2k4.GetValue("cmi.score.max"), api2k4.GetValue("cmi.score.scaled"), api2k4.GetValue("cmi.progress_measure"));
                    return Object.getPrototypeOf(api2k4).Commit.call(api2k4, p);
                };
                window.API_1484_11 = api2k4;
            }
            return (api12 || api2k4) != null;
        }
        catch (e) {
            console.log(e);
        }
        return false;
    }
    saveGlobalScore(resId, min, raw, max, scaled, progress) {
        if (!raw && scaled) {
            min = "-1";
            max = "1";
            raw = scaled;
        }
        if (!raw)
            return;
        const r = parseFloat(raw);
        const m = parseFloat(min);
        const M = parseFloat(max);
        const c = progress ? parseFloat(progress) : undefined;
        if (isNaN(m) || isNaN(r) || isNaN(M) || m >= M || r < m || r > M) {
            console.log("invalid SCORM score [min, raw, max]: ", min, raw, max);
            return;
        }
        this.main.scores.setScoreScorm(resId, m, r, M, c >= 0 && c <= 1 ? c : undefined);
    }
    buildMsgNoStorageAvailable(msgRoot) {
        if (this.main.real.scorm.buildMsgNoStorageAvailable) {
            this.main.real.scorm.buildMsgNoStorageAvailable(msgRoot);
        }
        else {
            msgRoot.append("Aucune donnée n'est enregistrée");
        }
        return msgRoot;
    }
    get isScormSynthAvailable() { var _a; return ((_a = this.main.real.scorm) === null || _a === void 0 ? void 0 : _a.fetchScormSynthFor) != null; }
    fetchScormSynthFor(resId, dataToFill) {
        return this.main.real.scorm.fetchScormSynthFor(resId, dataToFill);
    }
}
const LS_RendLocalStore = "par.rend.localStore";
function getDb() {
    if (db == null) {
        try {
            const openReq = window.indexedDB.open(`par:${location.pathname}`, 2);
            openReq.onupgradeneeded = (ev) => {
                const d = openReq.result;
                if (ev.oldVersion < 1) {
                    d.createObjectStore(DB_rendFields, { keyPath: ['rendId', 'fieldKey'] });
                    d.createObjectStore(DB_rendValid, { keyPath: ['rendId'] });
                    d.createObjectStore(DB_resData, { keyPath: ['resId'] });
                }
                if (ev.oldVersion < 2) {
                    d.createObjectStore(DB_scores, { keyPath: ['id'] });
                }
            };
            db = doRequest(openReq).catch((e) => null);
        }
        catch (e) {
            db = Promise.resolve(null);
        }
    }
    return db;
}
let db = null;
const DB_rendFields = "rendFields";
const DB_rendValid = "rendValid";
const DB_scores = "scores";
export const DB_resData = "resData";
export function doRequest(req) {
    let resolver = null;
    let reject = null;
    const p = new Promise((r, rej) => {
        resolver = r;
        reject = rej;
    });
    req.onsuccess = () => { resolver(req.result); };
    req.onerror = () => { reject(req.error); };
    return p;
}
export function doCommit(tr) {
    let resolver = null;
    let reject = null;
    const p = new Promise((r, rej) => {
        resolver = r;
        reject = rej;
    });
    tr.oncomplete = () => { resolver(); };
    tr.onerror = () => { reject(tr.error); };
    tr.commit();
    return p;
}
export function doCursor(req, cb, initValue) {
    let resolver = null;
    let reject = null;
    const p = new Promise((r, rej) => {
        resolver = r;
        reject = rej;
    });
    let lastV = initValue;
    req.onsuccess = (ev) => {
        const cursor = ev.target.result;
        if (cursor) {
            lastV = cb(cursor, lastV);
        }
        else {
            resolver(lastV);
        }
    };
    req.onerror = () => { reject(req.error); };
    return p;
}
const emptyFile = new File([], "");
export class StoredRendFieldsImpl extends Map {
    constructor(schema) {
        super();
        this.schema = schema;
    }
    setFieldString(fieldKey, content, ts, clientTs) {
        const field = this.schema.fields.find((f) => f.fieldId === fieldKey);
        if (field)
            this.set(fieldKey, new FieldString(fieldKey, field, content, ts, clientTs));
    }
    setFieldBlob(fieldKey, fileName, content, ts, clientTs) {
        const field = this.schema.fields.find((f) => f.fieldId === fieldKey);
        if (field)
            this.set(fieldKey, new FieldBlob(fieldKey, field, fileName, content, ts, clientTs));
    }
    getField(fieldKey) {
        return this.get(fieldKey);
    }
}
class FieldString {
    constructor(fieldKey, field, content, ts, clientTs) {
        this.fieldKey = fieldKey;
        this.field = field;
        this.content = content;
        this.ts = ts;
        this.clientTs = clientTs;
    }
    getString() {
        return this.content;
    }
    getBlob() { return null; }
}
class FieldBlob {
    constructor(fieldKey, field, fileName, content, ts, clientTs) {
        this.fieldKey = fieldKey;
        this.field = field;
        this.fileName = fileName;
        this.content = content instanceof Blob ? Promise.resolve(content) : content;
        this.ts = ts;
        this.clientTs = clientTs;
    }
    getString() { return null; }
    getBlob() {
        if (typeof this.content === "function")
            this.content = this.content();
        return this.content;
    }
}
export class BackendRendSynthFacade {
    constructor(front, main) {
        this.main = main;
    }
    isSynthRendAvailable() {
        var _a;
        return ((_a = this.main.real.rendSynth) === null || _a === void 0 ? void 0 : _a.canSynthRend) != null;
    }
    canSynthRend(schema) {
        var _a;
        return ((_a = this.main.real.rendSynth) === null || _a === void 0 ? void 0 : _a.canSynthRend(schema)) || { valid: false };
    }
    async fetchSynthRend(schema, fields, dataToFill) {
        var _a;
        if (dataToFill.length === 0)
            return;
        return (_a = this.main.real.rendSynth) === null || _a === void 0 ? void 0 : _a.fetchSynthRend(schema, true, fields, dataToFill);
    }
    async fetchSynthRendAddField(schema, field, dataToFill) {
        var _a;
        const fieldId = field.fieldId;
        const toFill = dataToFill.filter((u) => u.getField(fieldId) == null);
        if (toFill.length == 0)
            return;
        return (_a = this.main.real.rendSynth) === null || _a === void 0 ? void 0 : _a.fetchSynthRend(schema, false, [field], toFill);
    }
    async fetchSynthRendAllFields(schema, dataToFill) {
        const realSynth = this.main.real.rendSynth;
        if (realSynth) {
            if (realSynth.fetchSynthRendAllFields)
                return realSynth.fetchSynthRendAllFields(schema, dataToFill);
            if (realSynth.fetchSynthRend)
                return realSynth.fetchSynthRend(schema, false, schema.fields, [dataToFill]);
        }
    }
}
export class SynthUserImpl extends StoredRendFieldsImpl {
    constructor(schema, user) {
        super(schema);
        this.user = user;
    }
}
export class BackendScoresFacade {
    constructor(front, main) {
        this.scoreStoreMode = 'initing';
        this.front = front;
        this.main = main;
    }
    async waitScoresAvailable() {
        let map = this.scoresMap;
        for (;;) {
            if (map === undefined)
                return;
            await this.scoresMap;
            if (map === this.scoresMap)
                return;
            map = this.scoresMap;
        }
    }
    get fetchedScores() {
        return this.scoresMap instanceof Map ? this.scoresMap : undefined;
    }
    isScoresInSynthAvailable() { var _a; return ((_a = this.main.real.scores) === null || _a === void 0 ? void 0 : _a.fetchScoresFor) != null; }
    fetchScores() {
        this.scoresMap = undefined;
        this.scoreStoreMode = "initing";
        refreshScores(null);
        const scores = this.main.real.scores;
        if (!scores) {
            if (this.main.backendProps.rendNeverLocalyStored !== true) {
                const promise = this.scoresMap = getDb().then(async (db) => {
                    if (db) {
                        const defsMod = await scoreDefsModule();
                        const scores = await doRequest(db.transaction(DB_scores).objectStore(DB_scores).getAll());
                        const map = new Map();
                        scores.forEach((s) => {
                            const def = defsMod.getScoreDef(s.id);
                            if (def) {
                                if (isScoreDefValuable(def) && isScoreDataComputable(s))
                                    s.s = s.s * (def.max - def.min) + def.min;
                                map.set(s.id, s);
                            }
                        });
                        return map;
                    }
                });
                promise.then((map) => {
                    if (this.scoresMap !== promise)
                        return;
                    this.scoresMap = map;
                    this.scoreStoreMode = "device";
                    refreshScores(map);
                }).catch((e) => {
                    if (this.scoresMap !== promise)
                        return;
                    console.log(e);
                    this.scoresMap = undefined;
                    this.scoreStoreMode = "none";
                    refreshScores(null);
                });
            }
            else {
                this.scoresMap = undefined;
                this.scoreStoreMode = "none";
            }
        }
        else {
            const promise = this.scoresMap = scores.fetchScores();
            promise.then((map) => {
                if (this.scoresMap !== promise)
                    return;
                this.scoresMap = map;
                this.scoreStoreMode = "server";
                refreshScores(map);
            }).catch((e) => {
                if (this.scoresMap !== promise)
                    return;
                this.scoresMap = undefined;
                this.scoreStoreMode = "none";
                console.log(e);
                refreshScores(null);
            });
        }
    }
    async fetchScoresFor(scoreIds, dataToFill) {
        var _a;
        return (_a = this.main.real.scores) === null || _a === void 0 ? void 0 : _a.fetchScoresFor(scoreIds, dataToFill);
    }
    async setScore(scoreId, score, otherUser) {
        const defsMod = await scoreDefsModule();
        if (!defsMod)
            return;
        const def = defsMod.getScoreDef(scoreId);
        if (!def)
            return;
        const backendScores = this.main.user == null ? null : this.main.real.scores;
        let scoresMap;
        if (otherUser) {
            if (!(backendScores === null || backendScores === void 0 ? void 0 : backendScores.isComputManaged)) {
                const d = { user: otherUser };
                await this.fetchScoresFor('*', [d]);
                scoresMap = d.scores;
            }
        }
        else {
            scoresMap = await this.scoresMap;
            if (!scoresMap)
                return;
        }
        const scoresToRefresh = new Map();
        const scoresToSend = (backendScores === null || backendScores === void 0 ? void 0 : backendScores.isComputManaged) ? null : computeOnChange(defsMod, def, score, scoresMap);
        if (backendScores) {
            if (scoresToSend) {
                scoresToSend.forEach((v, k) => scoresToRefresh.set(k.id, v));
                if (backendScores.setScores) {
                    const r = await backendScores.setScores(scoresToSend, otherUser);
                    if (r)
                        r.forEach((score, id) => scoresToRefresh.set(id, score));
                }
                else {
                    const promises = [];
                    scoresToSend.forEach((v, k) => promises.push(backendScores.setScore(k, v, otherUser)));
                    (await Promise.all(promises)).forEach((r) => {
                        if (r)
                            r.forEach((score, id) => scoresToRefresh.set(id, score));
                    });
                }
            }
            else {
                scoresToRefresh.set(def.id, score);
                const r = await backendScores.setScore(def, score, otherUser);
                if (r)
                    r.forEach((score, id) => scoresToRefresh.set(id, score));
            }
        }
        else {
            if (this.main.backendProps.scoreNeverLocalyStored !== true && !otherUser) {
                const db = await getDb();
                if (db) {
                    const tr = db.transaction(DB_scores, 'readwrite');
                    const store = tr.objectStore(DB_scores);
                    const ts = now();
                    function setScore(def, s) {
                        store.put({
                            id: def.id,
                            s: isScoreDefValuable(def) && isScoreDataComputable(s) ? Math.max(0, Math.min((s.s - def.min) / (def.max - def.min), 1)) : undefined,
                            c: s.c,
                            a: s.a,
                            ts: ts
                        });
                        scoresToRefresh.set(def.id, s);
                    }
                    if (scoresToSend)
                        scoresToSend.forEach((v, k) => setScore(k, v));
                    else
                        setScore(def, score);
                    await doCommit(tr);
                }
            }
        }
        if (scoresToRefresh.size === 0) {
            if (scoresToSend)
                scoresToSend.forEach((v, k) => scoresToRefresh.set(k.id, v));
            else
                scoresToRefresh.set(scoreId, score);
        }
        if (!otherUser)
            scoresToRefresh.forEach((v, k) => scoresMap.set(k, v));
        this.front.scoresUpdated(scoresToRefresh, otherUser);
    }
    async setScoreScorm(resId, min, s, max, c) {
        try {
            const scoreDefMod = await scoreDefsModule();
            if (!scoreDefMod)
                return;
            const scores = scoreDefMod.getScoresForScorm().get(resId);
            if (scores)
                scores.forEach((scoreId) => {
                    const def = scoreDefMod.getScoreDef(scoreId);
                    const score = newScore(min, s, max, def.scoProgressAsCompletion ? c : undefined);
                    normScale(score, def.min, def.max, def.normAroundZero);
                    fixupScore(score);
                    const old = this.fetchedScores.get(scoreId);
                    if (old && old.s === score.s && old.c === score.c)
                        return;
                    this.setScore(def.id, score);
                });
        }
        catch (e) {
            DIALOG.confirm("La note finale de la ressoure n'a pu être enregistrée. Merci de tenter d'ouvrir et refermer ultérieurement cette ressource.");
        }
    }
}
